#include <mega128.h>

#ifndef DEF_H
#define DEF_H

//
#define trim_0up   PINA.3	//  1 
#define trim_0down PINA.2	//  1   (PINA.0  MSV)
#define trim_1up   PINA.1	//  2 
#define trim_1down PINA.0	//  2   (PINA.2  MSV)
#define trim_2up   PINA.7	//  3  (PINA.5  MSV)
#define trim_2down PINA.6	//  3   (PINA.4  MSV)
#define trim_3up   PINA.5	//  4  (PINA.7  MSV)
#define trim_3down PINA.4	//  4   (PINA.6  MSV)

// 
//#define SE_Joy

#ifdef SE_Joy // SonyEricsson joystic navigation
    #define L_UP    PIND.4
    #define L_DOWN  PIND.5          
    #define R_DOWN  PIND.6
    #define R_UP    PIND.7
    #define J_ENT   (PING & 1)
    #define KEY_UP    !(!L_UP && !R_UP && L_DOWN && R_DOWN) 
    #define KEY_DOWN  !(L_UP && R_UP && !L_DOWN && !R_DOWN)          
    #define KEY_LEFT  !(!L_UP && R_UP && !L_DOWN && R_DOWN)
    #define KEY_RIGHT !(L_UP && !R_UP && L_DOWN && !R_DOWN)
    #define KEY_ENTER (!J_ENT && L_UP && R_UP && L_DOWN && R_DOWN)    
#else // 4-button navigation - classic variant 
    #define KEY_UP    PIND.4
    #define KEY_DOWN  PIND.5          
    #define KEY_LEFT  PIND.6
    #define KEY_RIGHT PIND.7
#endif


//

#define BUZ0 PORTE&=0b11110111  // PORTE.3=0
//
#define LED1 PORTB.0

//
#define MODE_KEY1       PINE.6
#define MODE_KEY2       PINE.7    
#define SW1             PINE.4  
#define SW2_1           PINE.1
#define SW2_2           PINE.2
#define SW3             PINE.5

#define Tcut_KEY        PINC.3     
#define DUAL_ail        PINC.1     
#define DUAL_elev       PINC.0  
#define DUAL_rud        PINC.2     

//   - Mode 1 (?) -   MSV
/*
#define ch_Ailerons        6
#define ch_Elevator        5
#define ch_Throttle        4
#define ch_Rudder          3
#define ch_Rpop_contr      2
#define ch_Batt            7
#define ch_Antenna         0
#define ch_Temperature     1
*/

//   - Mode2 ("Clouddancer")
#define ch_Ailerons        5
#define ch_Elevator        4
#define ch_Throttle        6
#define ch_Rudder          3
#define ch_Rpop_contr      2
#define ch_Batt            7
#define ch_Temperature     1
#ifdef AntennaCtrlUse 
    #define ch_Antenna         0
    //#define ch_Power           1
#endif

//     
//    acntr[]! ( acntr[]   Coder.c,   "void Calc (void)")
#define AD_Ail          0
#define AD_Elev         1
#define AD_Thrtl        2
#define AD_Rud          3
#define AD_AdRpop       4
#define AD_Ghost        5    // ,    
                        
#define	MAX_MODES		3
#define MAX_CHANS		8
#define MAX_CONTRS		10
#define CURVE_NODES		7
#define MAX_MODELS      7

#define SIZE_RPM_BUFF 3

//   
typedef struct {
    unsigned char       minRates;        // EPA val in 2% unit
    unsigned char       maxRates;        // EPA val in 2% unit
    unsigned char       minDRates;       // EPA val in 2% unit
    unsigned char       maxDRates;        // EPA val in 2% unit
    char                cur_nodes[CURVE_NODES+1];
}CONTR_A1;
//  
typedef struct {
	unsigned char		minRates;	  // EPA val in 1% unit
	unsigned char		maxRates;	  // EPA val in 1% unit
	unsigned char		tcut;		  // EPA val in 1% unit
	char		        cur_nodes[CURVE_NODES+1];
}CONTR_A2;
// AddProp
typedef struct {
	unsigned char		minRates;	  // EPA val in 1% unit
	unsigned char		maxRates;	  // EPA val in 1% unit
	char		        cur_nodes[CURVE_NODES+1];
}CONTR_A3;
//  
typedef struct {
	unsigned char		minRates;	  // EPA val in 1% unit
	unsigned char		maxRates;	  // EPA val in 1% unit
    unsigned char       delay;        // in 0.1 sec
}CONTR_D;
//   
typedef struct {
	unsigned char		minRates;	  // EPA val in 1% unit
	unsigned char		maxRates;	  // EPA val in 1% unit
    unsigned char       from;         // ind controls
  	char		        cur_nodes[CURVE_NODES+1];
}CONTR_V;
// MODE 
typedef struct {
  CONTR_A1 contr_ab[3]; //      
  CONTR_A2 contr_tr; // 
  CONTR_A3 contr_ap; // . 
  CONTR_D  contr_d[3]; //  
  CONTR_V  contr_v[2]; //  
  signed char mixer[MAX_CHANS][MAX_CONTRS];
  signed char trimmers[4];
}DEF_MODE;
// Channels
typedef struct {
  signed char subtrimmer;
  signed char epa[2]; 
  signed char reverse;
}CHANNEL;
// Model   
typedef struct {
  char name[8]; //  0- !
  unsigned char type:1; // heli, plane.
  unsigned char modulation:2; // PPM, IPPM, PCM
  unsigned char timer_mode:2;
  unsigned int  timer_set;
  unsigned char num_ch:4;
  unsigned char num_freq;
  DEF_MODE modes[MAX_MODES];
  CHANNEL  channels[MAX_CHANS];
} MODEL_SET;

struct typedef { unsigned char note; unsigned int ms; }BEEP_TRACK_DEF;

#endif


